#include "stdio.h"
#include "stdlib.h"
#include "math.h"
#include "stm8s.h"
#include "usrlib-uart.h"

#define fMASTER  16000000  // 16MHz

void UART_Menu(void);

void main(void)
{

	// clock setting
	CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO, CLK_SOURCE_HSE, DISABLE, CLK_CURRENTCLOCKSTATE_DISABLE);
  
	// display indicator
	GPIO_Init(GPIOD, GPIO_PIN_0, GPIO_MODE_OUT_PP_HIGH_FAST);
  
  // frequency counter input port  TIM1_CH1 PC1
	TIM1_DeInit();
	TIM1_PrescalerConfig(0, TIM1_PSCRELOADMODE_UPDATE);
	TIM1_PWMIConfig(TIM1_CHANNEL_1, TIM1_ICPOLARITY_RISING, TIM1_ICSELECTION_DIRECTTI, TIM1_ICPSC_DIV1, 0);
	TIM1_SelectInputTrigger( TIM1_TS_TI1FP1 );
	TIM1->SMCR = (TIM1->SMCR & ~TIM1_SMCR_SMS) | 0b100; // TIM1->SMCR->SMS=100 (Reset mode)
  TIM1_ITConfig(TIM1_IT_CC1, ENABLE);
  TIM1_Cmd(ENABLE);

	// 1kHz 30% duty oscillator
	// 1kHz ---> 16MHz/1000 = 16000
	// 30% duty ---> 16000*0.3 = 4800
  // TIM2_CH1 D4 _______|~~~~~~|____________|~~~~~~|_______
  // TIM2_CH2 D3 ~~~~~~~|______|~~~~~~~~~~~~|______|~~~~~~~
	TIM2_DeInit();
	TIM2_TimeBaseInit(TIM2_PRESCALER_1, 15999);
	TIM2_OC1Init(TIM2_OCMODE_PWM1, TIM2_OUTPUTSTATE_ENABLE, 4800, TIM2_OCPOLARITY_HIGH); // output TIM2_CH1 PD4
	TIM2_OC2Init(TIM2_OCMODE_PWM2, TIM2_OUTPUTSTATE_ENABLE, 4800, TIM2_OCPOLARITY_HIGH); // output TIM2_CH2 PD3
	TIM2_Cmd(ENABLE);

  // display timebase
	TIM3_DeInit();
  TIM3_TimeBaseInit(TIM3_PRESCALER_1024 , 10000); // 16MHz/1024/10000=1.56Hz
  TIM3_ITConfig(TIM3_IT_UPDATE, ENABLE);
  TIM3_Cmd(ENABLE);
	
	UART2_DeInit();
  UART2_Init((u32)115200, UART2_WORDLENGTH_9D, UART2_STOPBITS_1, UART2_PARITY_ODD, UART2_SYNCMODE_CLOCK_DISABLE, UART2_MODE_TXRX_ENABLE);	
  UART2_ITConfig(UART2_IT_RXNE_OR, ENABLE);
	UART2_Cmd(ENABLE);
	
	enableInterrupts();
	
	SerialPutString("start\r\n");
	
  while (1)
	{
	  UART_Menu();
	}
}

// TIM1 interrupt routine
// get frequency counter values
//  __________|~~~~~~|____________|~~~~~~|__________
//            <------------------->  TIM1_ICvalue1
//            <------>   TIM1_ICvalue2
u16 TIM1_IC_value1=0, TIM1_IC_value2=0;
void TIM1_IC_isr(void) interrupt 12
{
	TIM1_IC_value1 = TIM1_GetCapture1()+2;  // get value
	TIM1_IC_value2 = TIM1_GetCapture2()+2;  // get value
	TIM1_ClearFlag(TIM1_FLAG_CC1);
}

// TIM3 interrupt routine
// display counter values
float OSC_HZ=1000.0, OSC_DUTY=33.3, ARR;
u8  TIM1_clk_prescale_calibrate_request;
u16 TIM1_clk_prescale;
u16 TIM2_clk_prescale;
char TXStr[100];
u8 verbose=0; // 1:verbose 0:normal
void TIM3_Update_isr(void) interrupt 15
{
  float MEASURED_HZ, MEASURED_DUTY;
	GPIO_WriteReverse(GPIOD,GPIO_PIN_0); // LED indicator
	
	// display frequency
	MEASURED_HZ = (float)fMASTER / (float)TIM1_clk_prescale / (float)TIM1_IC_value1;
	MEASURED_DUTY = (float)TIM1_IC_value2 / (float)TIM1_IC_value1 * 100.0 ;
	sprintf(TXStr,"OSC: %.2fHz %.1f%% ",OSC_HZ,OSC_DUTY);
	SerialPutString(TXStr);
	if(verbose)
	{
		sprintf(TXStr,"(PRS%u ARR%u) ",1<<TIM2_clk_prescale,(u16)ARR);
		SerialPutString(TXStr);
	}
	sprintf(TXStr,"-> MEAS: %.2fHz %.1f%% ",MEASURED_HZ,MEASURED_DUTY);
	SerialPutString(TXStr);
	if(verbose)
	{
		sprintf(TXStr,"(PRS%u %u/%u)",TIM1_clk_prescale,TIM1_IC_value2,TIM1_IC_value1);
		SerialPutString(TXStr);
	}
	SerialPutString("\r\n");
	
	// change prescaler
	if(TIM1_clk_prescale_calibrate_request==1)
	{
		if(TIM1_IC_value1>(u16)32000) TIM1_clk_prescale_calibrate_request=0; // calibration complete
		else
		{
			TIM1_clk_prescale=(u16)((float)TIM1_clk_prescale*0.9);
			if(TIM1_clk_prescale==0)
			{
				TIM1_clk_prescale=1;
				TIM1_clk_prescale_calibrate_request=0; // calibration complete
			}
			TIM1_PrescalerConfig(TIM1_clk_prescale-1, TIM1_PSCRELOADMODE_UPDATE); // change prescaler
		}
	}

	TIM3_ClearFlag(TIM3_FLAG_UPDATE);
}

int First=1;  // only 1st time
void UART_Menu(void)
{
	u16 duty;

	// command process
	if(UART_STR_EXIST==1 || First==1)
	{
		int fieldnumber;
		float a;
		First=0;
		
		// command parse
		fieldnumber = separate_line(command);
		if(fieldnumber==0) goto OSC;
		else if(Field[0][0]=='f' && fieldnumber==2){a=atof(Field[1]);if(a<10 || 200000<a)goto NG; OSC_HZ=a; goto OSC; }
		else if(Field[0][0]=='d' && fieldnumber==2){a=atof(Field[1]);if(a<1 || 99<a)goto NG; OSC_DUTY=a; goto OSC; }
		else if(Field[0][0]=='v' && fieldnumber==1){if(verbose==0)verbose=1;else verbose=0; goto RET;}
		else goto NG;

	OSC:
		// oscillator setting
		// output + TIM2_CH1  PD4
		// output - TIM2_CH2  PD3
		TIM2_clk_prescale=0;
		while(1){ // prescale and ARR decision
			int i;
			ARR=fMASTER/OSC_HZ;
			for(i=1;i<=TIM2_clk_prescale;i++) ARR/=2;
			if(ARR>65500) TIM2_clk_prescale++;
			else break;
		}
		duty=(u16)(ARR*OSC_DUTY/100.0);
		TIM2_DeInit();
		TIM2_TimeBaseInit(TIM2_clk_prescale, (u16)ARR-1);  // 16MHz/prescale/arr
		TIM2_OC1Init(TIM2_OCMODE_PWM1, TIM2_OUTPUTSTATE_ENABLE, duty, TIM2_OCPOLARITY_HIGH); // output PD4
		TIM2_OC2Init(TIM2_OCMODE_PWM2, TIM2_OUTPUTSTATE_ENABLE, duty, TIM2_OCPOLARITY_HIGH); // output PD3
		TIM2_Cmd(ENABLE);
		
		// request of frequency counter calibration
		TIM1_clk_prescale=25;  // prescaler start value
		TIM1_clk_prescale_calibrate_request=1; // start frequency counter calibration
		TIM1_PrescalerConfig(TIM1_clk_prescale-1, TIM1_PSCRELOADMODE_UPDATE);

	RET:	// menu display
 		SerialPutString("f 20.4 ---> 20.4 Hz (10-200000)\r\n");
		SerialPutString("d 10.0-99.0 ---> duty percent (1-99)\r\n");
		SerialPutString("v ---> verbose mode\r\n\n");
		UART_STR_EXIST=0;
		return;

	NG:
		SerialPutString("COMMAND NG!\r\n");
		goto RET;
	}
}

#ifdef USE_FULL_ASSERT
void assert_failed(u8* file, u32 line)
{ 
  while (1)  	{	}
}
#endif
